import axinite as ax
from tools import AxiniteArgs, to_vec, Body, string_to_color
from vpython import *
from itertools import cycle
import vpython as vp

colors = cycle([color.red, color.blue, color.green, color.orange, color.purple, color.yellow])

def live(limit, delta, t, *bodies: Body, radius_multiplier=1, rate=100, retain=200, name=None):
    if rate is None:
        rate = 100
    if radius_multiplier is None:
        radius_multiplier = 1
    if retain is None:
        retain = 200

    global pause
    pause = False

    scene = canvas()
    scene.select()
    if name: scene.title = name

    def pause_fn():
        global pause
        pause = not pause

    pause_btn = button(bind=pause_fn, text='Pause', pos=scene.caption_anchor)

    spheres = {}
    labels = {}
    lights = {}

    for body in bodies:
        body_color = string_to_color(body.color) if body.color != "" else next(colors)
        body_retain = body.retain if body.retain != None else retain
        spheres[body.name] = sphere(pos=to_vec(body.r[0]), radius=body.radius.value * radius_multiplier, color=body_color, make_trail=True, retain=body_retain, interval=10)
        labels[body.name] = label(pos=spheres[body.name].pos, text=body.name, xoffset=15, yoffset=15, space=30, height=10, border=4, font='sans')
        if body.light == True: lights[body.name] = local_light(pos=to_vec(body.r[0]), color=body_color)

    def fn(t, **kwargs):
        vp.rate(rate)
        for body in kwargs["bodies"]:
            spheres[body.name].pos = to_vec(body.r[t.value])
            labels[body.name].pos = spheres[body.name].pos
            try: lights[body.name].pos = spheres[body.name].pos
            except: pass
        print(f"t = {t}", end='\r')
        if pause: 
            while pause: vp.rate(10)

    ax.load(delta, limit, fn, *bodies, t=t)


import axinite as ax
from vpython import *
from tools import to_vec, to_float, string_to_color
from itertools import cycle
import vpython as vp

colors = cycle([color.red, color.blue, color.green, color.orange, color.purple, color.yellow])

def show(limit, delta, *bodies: axtools.Body, radius_multiplier=1, rate=100, retain=200, name=None):
    if rate is None:
        rate = 100
    if radius_multiplier is None:
        radius_multiplier = 1
    if retain is None:
        retain = 200

    global pause
    pause = False
    
    scene = canvas()
    scene.select()
    if name: scene.title = name

    def pause_fn():
        global pause
        pause = not pause

    pause_btn = button(bind=pause_fn, text='Pause', pos=scene.caption_anchor)

    spheres = {}
    labels = {}
    
    for body in bodies:
        body_color = string_to_color(body.color) if body.color != "" else next(colors)
        body_retain = body.retain if body.retain != None else retain
        spheres[body.name] = sphere(pos=to_vec(body.r[0]), radius=body.radius.value * radius_multiplier, color=body_color, make_trail=True, retain=body_retain, interval=10)
        labels[body.name] = label(pos=spheres[body.name].pos, text=body.name, xoffset=15, yoffset=15, space=30, height=10, border=4, font='sans')
    
    t = to_float(0)
    while t < limit:
        vp.rate(rate)
        for body in bodies:
            spheres[body.name].pos = to_vec(body.r[t])
            labels[body.name].pos = spheres[body.name].pos
        t += delta
        print(f"t = {t}", end='\r')
        while pause: vp.rate(10)






import json
import numpy as np
import astropy.units as u
import axinite as ax
from astropy.coordinates import CartesianRepresentation

def load(delta: u.Quantity, limit: u.Quantity, action, *bodies: ax.Body, t:u.Quantity=0 * u.s):
    while t < limit:
        for body in bodies: 
            others = [b for b in bodies if b != body]
            f = CartesianRepresentation([0, 0, 0] * u.kg * u.m/u.s**2)
            for other in others: f += other.gravitational_force(body.r[t.value] - other.r[t.value], body.mass)
            a = f / body.mass
            v = body.v[t.value] + delta * a
            r = body.r[t.value] + delta * v

            body.r[t.value + delta.value] = r
            body.v[t.value + delta.value] = v
        t += delta
        action(t, limit=limit, bodies=bodies)
        
    return bodies


from astropy.coordinates import CartesianRepresentation
from astropy.units import Quantity
import astropy.units as u
import math
import numpy as np

def apply_to_vector(vector: CartesianRepresentation, function):
    return CartesianRepresentation([function(i) for i in vector.xyz])

def vector_to(vector: CartesianRepresentation, unit: Quantity):
    return apply_to_vector(vector, lambda i: i.to(unit))

def vector_magnitude(vector: CartesianRepresentation):
    return np.sqrt(vector.x**2 + vector.y**2 + vector.z**2)

def unit_vector(vector: CartesianRepresentation):
    return vector / vector_magnitude(vector)

def to_vector(data, unit):
    return CartesianRepresentation(data["x"] * unit, data["y"] * unit, data["z"] * unit)


from axtools.args import AxiniteArgs
from axtools.body import Body
from axtools.functions import *
from axtools.load import load
from axtools.read import read
from axtools.show import show
from axtools.live import live